package samplepackage;

import java.util.HashMap;
import java.util.Map;

import biz.papercut.pcng.ext.paymentgateway.GatewayUtils;
import biz.papercut.pcng.plugin.TopupAccountOnDemandPlugin4;

/**
 * A simple demonstration top-up on demand gateway implementation showing how the PaperCut 
 * gateway interfaces should be used to build a top-up on demand gateway.
 * 
 * A top-up on demand gateway supports micro-transactions to an external online credit source to 
 * query user balance and obtain additional PaperCut credit for printing/copying.
 * 
 * A real-world information will need also to manage:
 *  - All communications with the online credit source including required security authentication.
 *  - Any required mapping between the PaperCut user and credit source account ID
 *  - Error handling, including handling of network errors, retries etc. 
 *  - Logging of transactions and errors
 *  - Ensuring any 3rd party license requirements are met
 *  - Reading the configuration from a configuration file and adjusting behavior accordingly.
 *    A configuration typically includes items such as:
 *      * whether the plugin is enabled. (required as this is an EnableablePlugin)
 *      * the merchant account id for the credit source,
 *      * whether the gateway is in test or production mode.
 *      * the min and max top-up amounts etc.
 *  
 * @author geoff
 *
 */
public class SampleGateway implements TopupAccountOnDemandPlugin4 {
	
	// The gateway may define a minimum and maximum transfer amount.  These may be made configurable.
	private static final double _minAmount = 0.50;
	private static final double _maxAmount = 10.00;
	
	private static final double _roundingTolerance = 0.00001;
	
	/**
	 * Constructor
	 */
	public SampleGateway() {
		
	}
	
	/**
	 * Returns {@code true} if this plugin is enabled
	 * The normal mechanism is to have a top level flag in the gateway's config file
	 * to specify enabled=Y|N.  
	 * This gateway is responsible for loading the config file.
	 */
	@Override
	public boolean isPluginEnabled() {
		return true;
	}
	
	/**
     * Calls the online service to get the available credit.  
     * See detailed comments in TopupAccountOnDemandPlugin3.java
     */
	@Override
	public AvailableCredit getAvailableCreditForTopUp(String username,
			String sharedAccount, AccountType accountCharged) {

		return new AvailableCredit(false, 10);
	}

	/**
	 * Perform the topup operation
	 */
	@Override
	public TopupResponse getTopup(double amountRequired, String username,
			String sharedAccount, AccountType accountCharged,
			Map<String, String> usageDetails) {
        if (!isPluginEnabled()) {
            return null;
        }

        // not enabled for shared accounts
        if (accountCharged == AccountType.SharedAccount) {
            System.out.println("This top-up on demand plugin is not enabled for shared accounts.");
            return null;
        }

        double topupAmount = calcTopupAmount(amountRequired);
        if (topupAmount <= 0.0) {
            // nothing to do
            return null;
        }

        String txnComment = "Prepare a suitable transaction comment to be logged in PaperCut";
        
        return GatewayUtils.performTopupDebit(this, 
                new GatewayUtils.TopUpDebitFunction() {
                    public Double performDebit(double amount) {
                        
                    	// Synchronously call the online service to perform the debit transaction
                    	// and validate the result.
                    	
                    	// MyResult result = myonlineservice.performTopupDebit(username, amount);
                    	
                    	// If successful, return the amount debited. 
                    	//
                    	// if (result != null && result.status == OK) {
                    	//     return result.getAmount();
                    	// }
                    	
                    	return amount;
                        
                    	// If anything went wrong, log the error and return null.
                    	// GatewayUtils.LogHelper.logError(log, ");
                        // return null;
                    }
                }, 
                topupAmount, amountRequired, username, sharedAccount, accountCharged, usageDetails, txnComment);		
	}

	/**
	 * Calls the online service to get the available balance to display
	 * See detailed comments in TopupAccountOnDemandPlugin4.java
	 */
	@Override
	public Map<String, Double> getBalance(Map<String, String> userDetails) {
		Map<String, Double> map = new HashMap<String, Double>();
		
		// Synchronously call the online service to obtain the user's balance
		
		//String username = userDetails.get(USER_DETAILS_USERNAME);
		// double balance = myonlineservice.getBalance(username);
		
		// Stub
		double balance = 10.0; 

		map.put("balance", balance);  // The key "balance" is the string to display

		return map;
	}
	
    /**
     * Calculate the amount to transfer based on amount required and configured default and max amounts.
     *
     * If the transfer exceeds the configured maximum, then 0.0 is returned to indicate no top-up should occur.
     *
     * @param amountRequired The amount required.
     * @return The amount to top up, or 0.0 if the top-up should not occur (i.e. because it exceeds maximum).
     */
	private double calcTopupAmount(double amountRequired) {
        if (amountRequired <= 0) {
            return 0;
        }
                
        // Round the amount to the nearest cent, or cent equivalent
        int cents = currencyInLowestUnitRounded(amountRequired);
        double topupAmount = cents / 100.0;        

        if (_minAmount > 0 && amountRequired < _minAmount) {
            System.out.println("Using minimum transfer amount of " + _minAmount);
            topupAmount = _minAmount;
        }

        if (_maxAmount > 0 && topupAmount > _maxAmount) {
            System.out.println("Transfer amount of " + topupAmount + " exceeds maximum of " + _maxAmount);
            topupAmount = 0.0;
        }

        return topupAmount;
	}
	
    /**
     * Convert the given double value representing a decimal currency to an integer value representing whole cents (or equivalent).
     * The double value is rounded up to the nearest cent if required.
     *
     * @param dollars The amount in dollars/Krona etc.
     * @return The amount in cents, rounded up to the nearest cent.
     */
    public static int currencyInLowestUnitRounded(double dollars) {
        double dollarsToRound = dollars - _roundingTolerance;
        int cents = (int) Math.ceil(dollarsToRound * 100);
        return cents;
    }
}
